#!/usr/bin/env python
# encoding: utf-8
"""
warning.py

Prints a warning message to stderr

Created by Måns Magnusson on 2014-08-22.
Copyright (c) 2014 __MoonsoInc__. All rights reserved.
"""


from __future__ import print_function
from __future__ import unicode_literals


import sys
import os

def warning(*objs):
    """Prints the warning messages to std err"""
    print("WARNING: ", *objs, file=sys.stderr)
    
def main():
    print('This is a warning!')

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# encoding: utf-8
"""
variant_annotator.py

Annotates variants.

Creates batches and put them into a queue object.
The batches are dictionary objects with overlapping features where the feature id:s are keys and the values are dictionarys with variants.


Batch = {feature_1_id:{variant_1_id:variant_1_info, variant_2_id: variant_2_info}, feature_2_id:... }   

Created by Måns Magnusson on 2014-03-17.
Copyright (c) 2014 __MyCompanyName__. All rights reserved.
"""

from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import argparse
from datetime import datetime
from codecs import open
import genmod
try:
    import cPickle as pickle
except:
    import pickle

from pprint import pprint as pp

from interval_tree import interval_tree


class VariantAnnotator(object):
    """Creates parser objects for parsing variant files"""
    def __init__(self, variant_parser, batch_queue, gene_trees={}, exon_trees={}, phased=False, 
                    vep=False, whole_genes=False, verbosity=False):
        super(VariantAnnotator, self).__init__()
        self.variant_parser = variant_parser
        self.batch_queue = batch_queue
        self.individuals = self.variant_parser.individuals
        self.verbosity = verbosity
        self.phased = phased
        self.vep = vep
        self.whole_genes = whole_genes
        self.gene_trees  = gene_trees
        self.exon_trees = exon_trees
        self.chromosomes = []
        self.interesting_so_terms = {'transcript_ablation':0, 'splice_donor_variant':0, 
                    'splice_acceptor_variant':0,'stop_gained':0, 'frameshift_variant':0, 'stop_lost':0,
                    'initiator_codon_variant':0, 'inframe_insertion':0, 'inframe_deletion':0, 'missense_variant':0,
                    'transcript_amplification':0, 'splice_region_variant':0,'incomplete_terminal_codon_variant':0,
                    'synonymous_variant':0, 'stop_retained_variant':0, 'coding_sequence_variant':0}
        
    
    def annotate(self):
        """Start the parsing"""        
        beginning = True
        batch = {}
        new_chrom = None
        current_chrom = None
        current_features = []
        haploblock_id = 1
        # Haploblocks is a dictionary with list of lists like {ind_id:[[start, stop, id],[start, stop,id],...], ...}
        haploblocks = {ind_id:[] for ind_id in self.individuals}
        nr_of_batches = 0
        # Parse the vcf file:
        if self.verbosity:
            start_parsing_time = datetime.now()
            start_chrom_time = start_parsing_time
            start_twenty_time = start_parsing_time
            if self.batch_queue.full():
                genmod.warning('Queue full!!')
            
        nr_of_variants = 0
        nr_of_comp_cand = 0
        for variant in self.variant_parser:
            # Only metadata lines start with '#'
            self.annotate_variant(variant)
            new_chrom = variant['CHROM']
            nr_of_variants += 1
            if variant['comp_candidate']:
                nr_of_comp_cand += 1
            new_features = variant['Annotation']
            if self.verbosity:
                if nr_of_variants % 20000 == 0:
                    print('%s variants parsed!' % nr_of_variants)
                    print('Last 20.000 took %s to parse.\n' % str(datetime.now() - start_twenty_time))
                    start_twenty_time = datetime.now()
            
            # If we look at the first variant, setup boundary conditions:
            if beginning:
                current_features = new_features
                beginning = False
                # Add the variant to each of its features in a batch
                batch = self.add_variant(batch, variant, new_features)
                current_chrom = new_chrom
                batch['haploblocks'] = {}
                if self.phased:
                    haploblock_starts = {ind_id:int(variant['POS']) for ind_id in self.individuals}
            else:
                # If we should put the batch in the queue:
                send = True
                
                if self.phased:
                    for ind_id in self.individuals:
                        #A new haploblock is indicated by '/' if the data is phased
                        if '/' in variant.get(ind_id, './.'):
                        #If call is not passed we consider it to be on same haploblock(GATK recommendations)
                            if variant.get('FILTER', '.') == 'PASS':
                                haploblocks[ind_id].append([haploblock_starts[ind_id], int(variant['POS']) - 1,
                                                             str(haploblock_id)])
                                haploblock_id += 1
                                haploblock_starts[ind_id] = int(variant['POS'])
                
            # Check if we are in a space between features:
                if len(new_features) == 0:
                    if len(current_features) == 0:
                        send = False
            #If not check if we are in a region with overlapping genes
                elif len(set.intersection(set(new_features),set(current_features))) > 0:
                    send = False
                
                if new_chrom != current_chrom:
                    self.chromosomes.append(current_chrom)
                    # New chromosome means new batch
                    send = True
                    current_chrom = new_chrom
                    
                    if self.verbosity:
                        print('Chromosome %s parsed!' % current_chrom)
                        print('Time to parse chromosome %s' % str(datetime.now()-start_chrom_time))
                        start_chrom_time = datetime.now()
                    
                if send:
                    if self.phased:
                    # Create an interval tree for each individual with the phaing intervals 
                        for ind_id in self.individuals:
                            #Check if we have just finished an interval
                            if haploblock_starts[ind_id] != int(variant['POS']):                                        
                                haploblocks[ind_id].append([haploblock_starts[ind_id], int(variant['POS']), 
                                                            str(haploblock_id)])
                                haploblock_id += 1
                            batch['haploblocks'][ind_id] = interval_tree.IntervalTree(haploblocks[ind_id], 
                                                haploblocks[ind_id][0][0]-1, haploblocks[ind_id][-1][1]+1)
                        haploblocks = {ind_id:[] for ind_id in self.individuals}
                    # Put the job in the queue
                    self.batch_queue.put(batch)
                    nr_of_batches += 1
                    #Reset the variables
                    current_features = new_features
                    batch = self.add_variant({}, variant, new_features)
                    batch['haploblocks'] = {}
                else:
                    current_features = list(set(current_features) | set(new_features))
                    batch = self.add_variant(batch, variant, new_features) # Add variant batch
                
        
        self.chromosomes.append(current_chrom)
        
        if self.verbosity:
            print('Chromosome %s parsed!' % current_chrom)
            print('Time to parse chromosome %s \n' % str(datetime.now()-start_chrom_time))
            print('Variants parsed!')
            print('Time to parse variants:%s' % str(datetime.now() - start_parsing_time))
            print('Number of variants in variant file:%s' % nr_of_variants)
        
        if self.phased:
        # Create an interval tree for each individual with the phasing intervals
            for ind_id in self.individuals:
                #check if we have just finished an interval
                if haploblock_starts[ind_id] != int(variant['POS']):
                    haploblocks[ind_id].append([haploblock_starts[ind_id], int(variant['POS']), str(haploblock_id)])
                    haploblock_id += 1
                try:
                    batch['haploblocks'][ind_id] = interval_tree.IntervalTree(haploblocks[ind_id], 
                                                haploblocks[ind_id][0][0]-1, haploblocks[ind_id][-1][1]+1)
                except IndexError:
                    pass
        # for gene in batch:
        #     print(gene)
        #     pp(batch[gene])
        self.batch_queue.put(batch)
        nr_of_batches += 1
        return nr_of_batches
    
    def add_variant(self, batch, variant, features):
        """Adds the variant to the proper gene(s) in the batch."""
        variant_id = '_'.join([variant['CHROM'], variant['POS'], variant['REF'], variant['ALT']])
        # If we are in a region between features:
        if len(features) == 0:
            if len(batch) == 0:
                batch['-'] = {variant_id:variant}
            else:
                batch['-'][variant_id] = variant
        for feature in features:
            if feature in batch:
                batch[feature][variant_id] = variant
            else:
                batch[feature] = {variant_id:variant}
        return batch
    
    def check_vep_annotatiion(self, vep_entry):
        """docstring for check_vep_annotatiion"""
        ### VEP: Allele|Gene|Feature|Feature_type|Consequence|cDNA_position|CDS_position|Protein_position|Amino_acids|Codons|Existing_variation|EXON|INTRON|DISTANCE|STRAND|SYMBOL|SYMBOL_SOURCE|SIFT|PolyPhen|HGVSc|HGVSp
        ###
            # Different genes are separated by ','
        annotation = {}#Dict with genes that variant is included in
        comp_candidate = False
        for gene in vep_entry:
            annotation[gene] = 0
            if not comp_candidate:
                for so_term in vep_entry[gene]['Consequence'].split('&'):
                    if so_term in self.interesting_so_terms:
                        comp_candidate = True
        
        return list(annotation.keys()), comp_candidate
        
    
    def annotate_variant(self, variant):
        """Returns a annotated variant dictionary in the cmms format."""
        
        variant['comp_candidate'] = False
        
        #If annotated with vep we do not need to check interval trees
        if self.vep:
            variant['Annotation'], variant['comp_candidate'] = self.check_vep_annotatiion(variant.get('vep_info',{}))
            variant.pop('vep_info')
        
        else:
            alternatives = variant['ALT'].split(',')
            # When checking what features that are overlapped we use the longest alternative
            longest_alt = max([len(alternative) for alternative in alternatives])
            # Internally we never use 'chr' in the chromosome names:
            variant_chrom = variant['CHROM'].lstrip('chr')
            
            variant_interval = [int(variant['POS']), (int(variant['POS']) + longest_alt-1)]
            
            try:
                variant['Annotation'] = self.gene_trees[variant_chrom].find_range(variant_interval)
                
            except KeyError:
                if self.verbosity:
                    genmod.warning(''.join('Chromosome', variant_chrom, 'is not in annotation file!'))
                variant['Annotation'] = []
            
            #Check if exonic:
            try:
                if len(self.exon_trees[variant_chrom].find_range(variant_interval)):
                    variant['comp_candidate'] = True
            except KeyError:
                if self.verbosity:
                    genmod.warning(''.join('Chromosome', variant_chrom, 'is not in annotation file!'))
        
        if self.whole_genes:
            if len(variant['Annotation']) > 0:
                variant['comp_candidate'] = True
        
        return variant

def main():
    from multiprocessing import JoinableQueue
    from vcf_parser import vcf_parser
    parser = argparse.ArgumentParser(description="Parse different kind of pedigree files.")
    parser.add_argument('variant_file', 
                            type=str, nargs=1, 
                            help='A file with variant information.'
    )
    parser.add_argument('-an', '--annotation_file', 
                            type=str, nargs=1, 
                            help='A file with feature annotations.'
    )
    parser.add_argument('-at', '--annotation_type', 
                            type=str, nargs=1,default=['gene_pred'], 
                            choices=['gene_pred', 'bed', 'gtf', 'ccds'], 
                            help='Specify annotation type.'
    )
    parser.add_argument('-phased', '--phased', 
                            action="store_true", 
                            help='If variant file is phased.'
    )
    parser.add_argument('-gene', '--whole_gene', 
                            action="store_true", 
                            help='If whole genes shall be checked for compounds.'
    )
    parser.add_argument('-v', '--verbose', 
                            action="store_true", 
                            help='Increase output verbosity.'
    )
    parser.add_argument('-vep', '--vep', 
                            action="store_true", 
                            help='If variants are annotated with vep.'
    )
    
    args = parser.parse_args()
    infile = args.variant_file[0]
    
    gene_trees = {}
    exon_trees = {}
    
    my_vcf_parser = vcf_parser.VCFParser(infile)
    if not args.vep:
        if args.annotation_file:
            anno_file = args.annotation_file[0]
            if args.verbose:
                print('Parsing annotationfile...')
                start_time_annotation = datetime.now()
            file_name, file_extension = os.path.splitext(anno_file)
            zipped = False
            if file_extension == '.gz':
                zipped = True
                file_name, file_extension = os.path.splitext(file_name)
            
            my_anno_parser = annotation_parser.AnnotationParser(anno_file, args.annotation_type[0], zipped=zipped)
            gene_trees = my_anno_parser.gene_trees
            exon_trees = my_anno_parser.exon_trees
            if args.verbose:
                print('annotation parsed. Time to parse annotation: %s\n' % str(datetime.now() - start_time_annotation))
        else:
            annopath = os.path.join(os.path.split(os.path.dirname(genmod.__file__))[0], 'annotations/')
            gene_db = os.path.join(annopath, 'genes.db')
            exon_db = os.path.join(annopath, 'exons.db')
        
            try:
                with open(gene_db, 'rb') as f:
                    gene_trees = pickle.load(f)
                with open(exon_db, 'rb') as g:
                    exon_trees = pickle.load(g)
            except FileNotFoundError:
                print('You need to build annotations! See documentation.')
                pass
            
        
    # print(my_head_parser.__dict__)
    variant_queue = JoinableQueue()
    start_time = datetime.now()        
    my_anno_parser = VariantAnnotator(my_vcf_parser, variant_queue, args, gene_trees, exon_trees)
    nr_of_batches = my_anno_parser.annotate()
    for i in range(nr_of_batches):
        variant_queue.get()
        variant_queue.task_done()
    
    variant_queue.join()
    if args.verbose:
        print('Time to parse variants: %s ' % str(datetime.now()-start_time))

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
variant_printer.py

Print the variants of a results queue to a file.

Created by Måns Magnusson on 2013-01-17.
Copyright (c) 2013 __MyCompanyName__. All rights reserved.
"""

from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import multiprocessing

from codecs import open
from tempfile import NamedTemporaryFile
from pprint import pprint as pp
from genmod import warning

class VariantPrinter(multiprocessing.Process):
    """docstring for VariantPrinter"""
    def __init__(self, task_queue, temp_dir, head, verbosity):
        multiprocessing.Process.__init__(self)
        self.task_queue = task_queue
        self.verbosity = verbosity
        self.file_handles = {}
        self.temp_dir = temp_dir
        self.header = head.header
    
    def run(self):
        """Starts the printing"""
        # Print the results to a temporary file:
        number_of_finished = 0
        proc_name = self.name
        if self.verbosity:
            print(('%s: starting!' % proc_name))
        while True:
            next_result = self.task_queue.get()
            if self.verbosity:
                if self.task_queue.full():
                    warning('Printing queue full')
            if next_result is None:
                if self.verbosity:
                    print('All variants printed!')
                for chromosome in self.file_handles:
                    self.file_handles[chromosome].close()
                break
                
            else:
                for variant_id in next_result:
                    variant_chrom = next_result[variant_id]['CHROM']
                    print_line = [next_result[variant_id].get(entry, '-') for entry in self.header]
                    if variant_chrom in self.file_handles:
                        self.file_handles[variant_chrom].write('\t'.join(print_line) + '\n')
                    else:
                        temp_file = NamedTemporaryFile(prefix=variant_chrom+'_', dir=self.temp_dir, delete=False)
                        temp_file.close()
                        self.file_handles[variant_chrom] = open(temp_file.name, mode='w', encoding='utf-8', errors='replace')
                        self.file_handles[variant_chrom].write('\t'.join(print_line) + '\n')
        return
    
def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
file_sorter.py

Sort a variant file based on position or rank score.


Created by Tomasz Beiruta on 2005-05-31.

Modified by Måns Magnusson on 2014-01-14.

"""

from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import argparse
from tempfile import NamedTemporaryFile
from codecs import open

from genmod.is_number import is_number


class FileSort(object):
    def __init__(self, inFile, outfile=None, splitSize=20, silent=False):
        """ split size (in MB) """
        self._inFile = inFile
        self._silent = silent
                
        self._outFile = outfile
                    
        self._splitSize = splitSize * 1000000
                
        self._getKey = lambda variant_line: (int(variant_line.split('\t')[1]))
    
    def sort(self):
        
        files = self._splitFile()

        if files is None:
            """ file size <= self._splitSize """            
            self._sortFile(self._inFile, outFile=self._outFile, ready_to_print=True)
            return

        for fn in files:
            self._sortFile(fn)
            
        self._mergeFiles(files)
        self._deleteFiles(files)

        
    def _sortFile(self, fileName, outFile=None, ready_to_print=False):
        try:
            lines = open(fileName, mode='r', encoding='utf-8').readlines()
        except TypeError:
            lines = open(fileName.name, mode='r', encoding='utf-8').readlines()
        get_key = self._getKey
        data = [(get_key(line), line) for line in lines if line!='']
        data.sort()
        lines = [line[1] for line in data]
        if ready_to_print:
            if outFile:
                open(outFile, mode = 'a', encoding = 'utf-8').write(''.join(lines))
            else:
                if not self._silent:
                    for line in lines:
                        print(line.rstrip())
        else:
        # In this case the temporary files are over witten.
            try:
                f = open(fileName, mode='w', encoding='utf-8')
            except TypeError:
            #If we deal with temporary files:
                f = open(fileName.name, mode='w', encoding='utf-8')
            f.write(''.join(lines))
    
    def _splitFile(self):
        totalSize = os.path.getsize(self._inFile)
        if totalSize <= self._splitSize:
            # do not split file, the file isn't so big.
            return None

        fileNames = []
        with open(self._inFile, mode='r', encoding='utf-8') as f:
            size = 0
            lines = []
            for line in f:
                size += len(line)
                lines.append(line)
                if size >= self._splitSize:
                    temp_file = NamedTemporaryFile(delete=False)
                    temp_file.close()
                    with open(temp_file.name, mode='w', encoding='utf-8') as f:
                        fileNames.append(temp_file.name)
                        f.write(''.join(lines))
                        del lines[:]
                        size = 0
                                                       
            if size > 0:
                temp_file = NamedTemporaryFile(delete=False)
                temp_file.close()
                with open(temp_file.name, mode='w', encoding='utf-8') as f:
                    fileNames.append(temp_file.name)
                    f.write(''.join(lines))
            
        return fileNames

    def _mergeFiles(self, files):
        try:
            files = [open(f, mode='r', encoding='utf-8') for f in files]
        except TypeError:
            files = [open(f.name, mode='r', encoding='utf-8') for f in files]
            
        lines = []
        keys = []
        
        for f in files:
            l = f.readline()  
            lines.append(l)
            keys.append(self._getKey(l))
        
        buff = []
        buffSize = self._splitSize/2
        append = buff.append
        if self._outFile:
            output = open(self._outFile, mode='a', encoding = 'utf-8')
        try:
            key = max(keys)
            index = keys.index(key)
            get_key = self._getKey
            while 1:
                while key == max(keys):
                    append(lines[index])
                    if len(buff) > buffSize:
                        if self._outFile:
                            output.write(''.join(buff))
                        else:
                            if not self._silent:
                                print(''.join(buff))
                        del buff[:]
                            
                    line = files[index].readline()
                    if not line:
                        files[index].close()
                        del files[index]
                        del keys[index]
                        del lines[index]
                        break
                    key = get_key(line)
                    keys[index] = key
                    lines[index] = line
        
                if len(files)==0:
                    break
                # key != min(keys), see for new index (file)
                key = max(keys)
                index = keys.index(key)

            if len(buff)>0:
                if self._outFile:
                    output.write(''.join(buff))
                else:
                    if not self._silent:
                        print(''.join(buff))
        finally:
            if self._outFile:
                output.close()
    
    def _deleteFiles(self, files):   
        try:
            for fn in files:
                os.remove(fn)
        except TypeError:
            for fn in files:
                os.remove(fn.name)
    


def main():
    parser = argparse.ArgumentParser(description="Check files.")
    parser.add_argument('infile', type=str, nargs=1, help='Specify the path to the file of interest.')
    parser.add_argument('-out', '--outfile', type=str, nargs=1, default=[None], help='Specify the path to the outfile.')
    args = parser.parse_args()
    infile = args.infile[0]
    temp_file = NamedTemporaryFile(delete=False)
    temp_file.close()
    with open(infile, mode='r', encoding = 'utf-8') as f:
        with open(temp_file.name, mode='w', encoding = 'utf-8') as g:
            for line in f:
                if not line.startswith('#'):
                    g.write(line)
    print('no errors')
    fs = FileSort(temp_file.name, args.outfile[0])
    fs.sort()


if __name__ == '__main__':
    main()


#!/usr/bin/env python
# encoding: utf-8
"""
genetic_models.py

Genetic models take a family object with individuals and variants and annotates for each variant which models they follow in this family.

The following models are checked:

- Autosomal Dominant(AD)
- Autosomal Dominant De Novo(AD_DN)
- Autosomal Recessive(AR_hom)
- Autosomal Recessive De Novo(AR_DN)
- Autosomal Recesive Compound(AR_comp).

In this model a variant must imply affected status, otherwise it can not be dominant. All sick has to be ay least heterozygote for the variant and all healthy can not have it.

We will assume that each individual that we have information about is present among the individual in self.family.individuals.


is the individual sick?

    - If the individual is homozygote alternative then AD/AD-denovo and AR/AR-denovo are ok

    - If the individual is is heterozygote then AD/AD-denovo are ok but AR/AR-denovo are not ok

    - If the individual is homozygote reference no model is ok

    - If the individual has no call all models are ok



is the individual healthy?

    - If the individual is homozygote alternative then no model is ok

    - If the individual is heterozygote then AR/AR-denove are ok but AD/AD-denovo are not ok

    - If the individual is homozygote referense all models are ok

    - If there is no call all models are ok



is there no known phenotype?

    - All models are ok for all variants



Created by Måns Magnusson on 2013-02-12.
Copyright (c) 2013 __MyCompanyName__. All rights reserved.
"""

from __future__ import print_function
from __future__ import unicode_literals

import os
import sys
from datetime import datetime
from pprint import pprint as pp

from genmod import pair_generator

def check_genetic_models(variant_batch, family, verbose = False, phased = False, strict = False, proc_name = None):
    #A variant batch is a dictionary on the form {gene_id: {variant_id:variant_dict}}
    # Start by getting the genotypes for each variant:
    individuals = family.individuals.values()
    intervals = variant_batch.pop('haploblocks', {})
    for gene in variant_batch:
        
        compound_candidates = []
        compound_pairs = []
        for variant_id in variant_batch[gene]:
            variant = variant_batch[gene][variant_id]
            # pp(variant)
            variant['Compounds'] = {}
            # Add information of models followed:
            variant['Inheritance_model'] = {'XR' : False, 'XR_dn' : False, 'XD' : False, 
                                            'XD_dn' : False, 'AD' : False, 'AD_dn' : False, 
                                            'AR_hom' : False, 'AR_hom_dn' : False, 'AR_comp' : False, 
                                            'AR_comp_dn' : False
                                            }
            if gene != '-':
                if check_compound_candidates(variant, family, strict):
                    compound_candidates.append(variant_id)
            # Only check X-linked for the variants in the X-chromosome:
            # For X-linked we do not need to check the other models
            if variant['CHROM'] == 'X':
                if check_X_recessive(variant, family, strict):
                    variant['Inheritance_model']['XR'] = True
                    for individual in family.individuals:
                        if family.individuals[individual].has_parents:
                            check_parents('X_recessive', individual, family, variant)
                                
                if check_X_dominant(variant, family, strict):
                    variant['Inheritance_model']['XD'] = True
                    for individual in family.individuals:
                        if family.individuals[individual].has_parents:
                            check_parents('X_dominant', individual, family, variant)
            else:
                # Check the dominant model:
                if check_dominant(variant, family, strict):
                    variant['Inheritance_model']['AD'] = True
                    for individual in family.individuals:
                        if family.individuals[individual].has_parents:
                            check_parents('dominant', individual, family, variant)
                    
                # Check the recessive model:
                if check_recessive(variant, family, strict):
                    variant['Inheritance_model']['AR_hom'] = True
                    for individual in family.individuals:
                        if family.individuals[individual].has_parents:
                            check_parents('recessive', individual, family, variant)
                
    
        # Now check the compound models:
        if len(compound_candidates) > 1:
            
            compound_pairs = pair_generator.Pair_Generator(compound_candidates)
            
            for pair in compound_pairs.generate_pairs():
                variant_1 = variant_batch[gene][pair[0]]
                variant_2 = variant_batch[gene][pair[1]]
                # We know from check_compound_candidates that all variants are present in all affected
                
                if check_compounds(variant_1, variant_2, family, intervals, phased):
                    variant_1['Inheritance_model']['AR_comp'] = True
                    variant_2['Inheritance_model']['AR_comp'] = True
                    for individual in family.individuals:
                        if family.individuals[individual].has_parents:
                            check_parents('compound', individual, family, variant_1, variant_2)
                    variant_1['Compounds'][pair[1]] = 0
                    variant_2['Compounds'][pair[0]] = 0
                    
    return

def check_compound_candidates(variant, family, strict):
    """Sort out the variants that are potential compound candidates. 
        This function is used to reduce the number of potential candidates for the future analysis.
        It will go through all variants in a batch(gene or other feature) and filter out those variants that not fit the model.
        Returns a bool depending on if the variant is a potential compound candidate.
        
        Cases:
            Affected:
                - If individual is affected it needs to be heterozygpte otherwise it can not be a compound candidate
                
            Healthy:
                - Can not be hom. alt for any variant in a potential compound pair.
        
        If strict:
            Affected must be heterozygote
        
        Args:
            variant: A variant dictionary.
            family: A family object with information about the family members for this analysis
            
        Returns:
            bool: depending on if the variant is a potential compound candidate according to the
            rules stated above
        
    """
    # This is the case when the variant is located in an uninteresting region(non gene region):
    if not variant.get('comp_candidate',True):
        return False
    for individual in family.individuals:
        individual_genotype = variant['genotypes'][individual]
        # No individuals can be homo_alt
        if individual_genotype.homo_alt:
            return False
        if family.individuals[individual].affected:
            # Affected can not be hom. ref. for compounds
            if individual_genotype.homo_ref:
                return False
            if strict:
                if not individual_genotype.heterozygote:
                    return False
    
    return True

def check_compounds(variant_1, variant_2, family, intervals, phased):
    """Check if two variants of a pair follow the compound heterozygous model. 
        At this stage we know: 
            - None of the individuals are homozygote alternative for the variants
            - All affected individuals are heterozygote for both variants.
            
        We do not allow healthy individuals to be heterozygote for both variants in the pair(ref. 
        http://www.plosone.org/article/info%3Adoi%2F10.1371%2Fjournal.pone.0070151).
        
        If the individuals are phased we will only consider pairs that are on different alleles in affected individuals.
        
        Args:
            variant_1, variant_2: Variants in a potential compound pair
            family: A family object with the individuals
            intervals: A interval tree that describes the phased intervals
            phased: A bool that tells if the individuals are phased
        
        Returns:
            bool: depending on if the pair follow the rules stated above
            
    """
    # Check in all individuals what genotypes that are in the trio based of the individual picked.
    
    for individual in family.individuals:

        genotype_1 = variant_1['genotypes'][individual]
        genotype_2 = variant_2['genotypes'][individual]
        #check if variants are in the same phased interval:
        if phased:
            variant_1_interval = intervals[individual].find_range([int(variant_1['POS']),int(variant_1['POS'])])
            variant_2_interval = intervals[individual].find_range([int(variant_1['POS']),int(variant_1['POS'])])
        
        # We know that no incdividuals are homomzygote alternative
        if family.individuals[individual].healthy:
            if genotype_1.heterozygote and genotype_2.heterozygote:
                return False
        # The case where the individual is affected
        # We know since ealier that all affected are heterozygotes for these variants
        elif family.individuals[individual].affected:
            #If the individual is sick and phased it has to have one variant on each allele
            if phased:
                # Variants need to be in the same phased interval, othervise we will not know
                if variant_1_interval == variant_2_interval:
                # If they are in the same interval they can not be on same allele
                    if (genotype_1.allele_1 == genotype_2.allele_1) or (genotype_1.allele_2 == genotype_2.allele_2):
                        return False
    
    return True


def check_dominant(variant, family, strict):
    """Check if the variant follows the autosomal dominant (AD) pattern in this family.
        A variant is following the dominant patttern if:
        Healthy:
            - Can not have the variant in any form.
            - If no call we can not exclude dominant.
            if strict:
                - Have to be homozygote reference
                - No call will return false
        
        Affected:
            - Has to be heterozygote for this position.
            - If no call we can not exclude dominant.
            if strict:
                - Have to be heterozygote
                - No call will return false
        
        No affection status:
            We can not tell if variant follows the model or not.
        
        Args:
            variant: variant dictionary.
            family: A family object with the individuals
            strict: A boolean that tells if strict analyzis should be performed.
        
        Return:
            bool: depending on if the model is followed in these indivduals
        
            """
    
    for individual in family.individuals: 
        # Check in all individuals what genotypes that are in the trio based of the individual picked.
        individual_genotype = variant['genotypes'][individual]
        if strict:
            if not individual_genotype.genotyped:
                return False
        if family.individuals[individual].healthy:# The case where the individual is healthy
            if individual_genotype.has_variant:
                return False
        
        elif family.individuals[individual].affected:
            # The case when the individual is sick
            if individual_genotype.genotyped:
                if not individual_genotype.heterozygote:
                    return False
    
    return True

def check_recessive(variant, family, strict):
    """Check if the variant follows the autosomal recessive homozygote (AR_hom) pattern in this family.
        A variant is following the AR_hom pattern if:
        Healthy:
            - Can not be homozygote alternative.
            - If no call we can not exclude AR
            if strict:
                - Have to be homozygote reference or heterozygote.
                - No call will return False
        
        Affected:
            - Have to be homozygote alternative.
            - If no call we can not exclude AR
            if strict:
                - Have to be homozygote alternative
                - No call will return false
        
        No affection status:
            We can not tell if variant follows the model or not.
        
        Args:
            variant: variant dictionary.
            family: A family object with the individuals
            strict: A boolean that tells if strict analyzis should be performed.
        
        Return:
            bool: depending on if the model is followed in these indivduals
        
            """
    for individual in family.individuals:
        individual_genotype = variant['genotypes'][individual]
        if strict:
            if not individual_genotype.genotyped:
                return False
        # The case where the individual is healthy:
        if family.individuals[individual].healthy:
        # If the individual is healthy and homozygote alt the model is broken.
            if individual_genotype.genotyped:
                if individual_genotype.homo_alt:
                    return False
                
        # The case when the individual is sick:
        elif family.individuals[individual].affected:
        # In the case of a sick individual it must be homozygote alternative for Autosomal recessive to be true.
        # Also, we can not exclude the model if no call.
            if individual_genotype.genotyped:
                if not individual_genotype.homo_alt:
                    return False
    
    return True

def check_X_recessive(variant, family, strict):
    """Check if the variant follows the x linked heterozygous (XR) pattern of inheritance in this family.
        A variant is following the XR pattern if:
        
        Healthy:
            - Can not be homozygote alternative
            - If no call we can not exclude XR
            - Males can not have variant at all. This is added since sometimes males 
                get called as heterozygotes but this should not be possible since 
                they only have one copy of the X chromosome.
            if strict:
                - Have to be homozygote reference or heterozygote.
                - No call will return False
        
        Affected:
            - Have to be homozygote alternative.
            - If no call we can not exclude AR
            if strict:
                - Have to be homozygote alternative
                - No call will return false
        
        No affection status:
                We can not tell if variant follows the model or not.
        
        Args:
            variant: variant dictionary.
            family: A family object with the individuals
            strict: A boolean that tells if strict analyzis should be performed.
        
        Return:
            bool: depending on if the model is followed in these indivduals
        
            """
    for individual in family.individuals:
        # Get the genotype for this variant for this individual
        individual_genotype = variant['genotypes'][individual]
        
        if strict:
            if individual_genotype.genotyped:
                return False
        # The case where the individual is healthy
        if family.individuals[individual].healthy:
            # If individual is healthy and homozygote alternative the variant can not be deleterious:
            if individual_genotype.genotyped:
                if individual_genotype.homo_alt:
                    return False
                if family.individuals[individual].sex == 1:
                    if individual_genotype.has_variant:
                        return False
        
        # The case when the individual is sick
        elif family.individuals[individual].affected:
        #If the individual is sick and homozygote ref it can not be x-recessive
            if individual_genotype.genotyped:
                if individual_genotype.homo_ref:
                    return False
        # Women have to be hom alt to be sick (almost allways carriers)
                elif family.individuals[individual].sex == 2:
                    if not individual_genotype.homo_alt:
                        return False
    return True

def check_X_dominant(variant, family, strict):
    """Check if the variant follows the x linked dominant (XD) pattern of inheritance in this family.
        A variant is following the XD pattern if:
        
        Healthy:
            - Can not be homozygote alternative
            - Healthy females can be heterozygotes. This is possible since there are several
             documented diseases where only one allele is expressed at a time during development.
            - If no call we can not exclude XR
            if strict:
                - Have to be homozygote reference (or heterozygote womens).
                - No call will return False
        
        Affected:
            - Have to be heterozygote.
            - If no call we can not exclude AR
            if strict:
                - Have to be heterozygote
                - No call will return false
        
        No affection status:
                We can not tell if variant follows the model or not.
        
        Args:
            variant: variant dictionary.
            family: A family object with the individuals
            strict: A boolean that tells if strict analyzis should be performed.
        
        Return:
            bool: depending on if the model is followed in these indivduals
        
        """
    for individual in family.individuals:
        # Get the genotype for this variant for this individual
        individual_genotype = variant['genotypes'][individual]
        
        if strict:
            if individual_genotype.genotyped:
                return False
        # The case where the individual is healthy
        if family.individuals[individual].healthy:
        # Healthy womans can be carriers but not homozygote:
            if individual_genotype.genotyped:
                if family.individuals[individual].sex == 2:
                    if individual_genotype.homo_alt:
                        return False
                # Males can not carry the variant:
                elif family.individuals[individual].sex == 1:
                    if individual_genotype.has_variant:
                        return False
        
        # The case when the individual is sick
        elif family.individuals[individual].affected:
        #If the individual is sick and homozygote ref it can not be x-linked-dominant
            if individual_genotype.genotyped:
                if individual_genotype.homo_ref:
                    return False
    return True

def check_parents(model, individual, family, variant, variant_2={}, strict = False):
    """Check if information in the parents can tell us if model is de novo or not. 
        Model IN ['recessive', 'compound', 'dominant', 'X_recessive', 'X_dominant'].
        If the expected pattern of a variant is followed in the family, de novo will be False. 
        Otherwise de novo will be True.
        
        If only one parent is present then we can never exclude denovo for heterozygous inheritance patterns
        If strict and one parent we will never say it is denovo
         
         """
    sex = family.individuals[individual].sex
    individual_genotype = variant['genotypes'][individual]
    
    mother_genotype = False
    father_genotype = False
    
    parent_genotypes = []
    mother_id = family.individuals[individual].mother
    father_id = family.individuals[individual].father
    
    if mother_id != '0':
        mother_genotype = variant['genotypes'][mother_id]
        mother_phenotype = family.get_phenotype(mother_id)
        parent_genotypes.append(mother_genotype)
    
    if father_id != '0':
        father_genotype = variant['genotypes'][father_id]
        father_phenotype = family.get_phenotype(father_id)
        parent_genotypes.append(father_genotype)

    if model == 'recessive':
        # If a parent is homozygote or if both parents are heterozygote the variant is not denovo.
        # If strict we know from before that both parents are genotyped
        if len(parent_genotypes) == 2:
            if not (mother_genotype.has_variant and father_genotype.has_variant):
                variant['Inheritance_model']['AR_hom_dn'] = True
            # If both parents are called but none of the above is fullfilled it is pure denovo
                if mother_genotype.genotyped and father_genotype.genotyped:
                    variant['Inheritance_model']['AR_hom'] = False
        elif not strict:
            variant['Inheritance_model']['AR_hom_dn'] = True            
                    
    elif model == 'dominant':
        # If none of the parents carry variant it is de novo
        if len(parent_genotypes) == 2:
            if not (mother_genotype.has_variant or father_genotype.has_variant):
                variant['Inheritance_model']['AD_dn'] = True
        # If both parents are called but none of them carry the variant it is denovo
                if mother_genotype.genotyped and father_genotype.genotyped:
                    variant['Inheritance_model']['AD'] = False
        else:
            for parent in parent_genotypes:
                if not parent.has_variant:
                    variant['Inheritance_model']['AD_dn'] = True
                    variant['Inheritance_model']['AD'] = False
            
    elif model == 'X_recessive':
        #If the individual is a male we only need to check if the mother carry the variant:
        if sex == 1:
            if mother_genotype:
                if not mother_genotype.has_variant:
                    variant['Inheritance_model']['XR_dn'] = True
                    if mother_genotype.genotyped:
                        variant['Inheritance_model']['XR'] = False
            elif not strict:
                variant['Inheritance_model']['XR_dn'] = True
                
        #If female, both parents must have the variant otherwise denovo is true
        elif sex == 2:
            if len(parent_genotypes) == 2:
                if not (mother_genotype.has_variant and father_genotype.has_variant):
                    variant['Inheritance_model']['XR_dn'] = True
        #If both parents are genotyped but they both are not carriers XR is not true
                    if (mother_genotype.genotyped and father_genotype.genotyped):
                        variant['Inheritance_model']['XR'] = False
            elif not strict:
                variant['Inheritance_model']['XR_dn'] = True
                
    elif model == 'X_dominant':
        #If the individual is a male we only need to look at the mother:
        if sex == 1:
            if mother_genotype:
                if not mother_genotype.has_variant:
                    variant['Inheritance_model']['XD_dn'] = True
                    if mother_genotype.genotyped:
                        variant['Inheritance_model']['XD'] = False
        #If female, one of the parents must have the variant otherwise denovo is true
        elif sex == 2:
            if len(parent_genotypes) == 2:
                if not (mother_genotype.has_variant or father_genotype.has_variant):
                    variant['Inheritance_model']['XD_dn'] = True
                    if mother_genotype.genotyped and father_genotype.genotyped:
                        variant['Inheritance_model']['XD'] = False
            elif not strict:
                variant['Inheritance_model']['XD_dn'] = True
    
    elif model == 'compound':
        individual_genotype_2 = variant_2['genotypes'][individual]
        mother_genotype_2 = None
        father_genotype_2 = None
        parent_genotypes_2 = []
        
        if mother_id != '0':
            mother_genotype_2 = variant_2['genotypes'][mother_id]
            parent_genotypes_2.append(mother_genotype_2)
        if father_id != '0':
            father_genotype_2 = variant_2['genotypes'][father_id]
            parent_genotypes_2.append(father_genotype_2)
        # One of the variants must come from father and one from mother
        if (len(parent_genotypes) == 2 and len(parent_genotypes_2) == 2):
            if not (mother_genotype.has_variant or mother_genotype_2.has_variant):
                variant['Inheritance_model']['AR_comp_dn'] = True
                variant_2['Inheritance_model']['AR_comp_dn'] = True
                if mother_genotype.genotyped and mother_genotype_2.genotyped:
                    variant['Inheritance_model']['AR_comp'] = False
                    variant_2['Inheritance_model']['AR_comp'] = False
            if not (father_genotype.has_variant or father_genotype_2.has_variant):
                variant['Inheritance_model']['AR_comp_dn'] = True
                variant_2['Inheritance_model']['AR_comp_dn'] = True
                if father_genotype.genotyped and father_genotype_2.genotyped:
                    variant['Inheritance_model']['AR_comp'] = False
                    variant_2['Inheritance_model']['AR_comp'] = False
        elif not strict:
            variant['Inheritance_model']['AR_comp_dn'] = True
            variant_2['Inheritance_model']['AR_comp_dn'] = True
            
    return
            
        

def main():
    from ped_parser import family, individual
    from interval_tree import interval_tree
    from vcf_parser import genotype
    
    duo_family = family.Family(family_id = '1')
    sick_son = individual.Individual(ind='1', family='1',mother='3', father='0', sex=1, phenotype=2)
    healthy_mother = individual.Individual(ind='3', family='1',mother='0', father='0', sex=2, phenotype=1)
    duo_family.add_individual(sick_son)
    duo_family.add_individual(healthy_mother)
    
    pp(duo_family.individuals)
    intervals = {ind_id:interval_tree.IntervalTree([[1,100, '1']], 1, 100) for ind_id in duo_family.individuals}

    pp(intervals)
    
    #Setup two variants with autosomal recessive compound pattern
    recessive_comp_simple_1 = {'CHROM':'1', 'POS':'5', 'ALT':'A', 'REF':'C', 'ID':'rs2230749',
                             '1':'0|1', '3':'0|0'}
    genotypes = {'1':genotype.Genotype('0|1'), '3':genotype.Genotype('0|0')}
    recessive_comp_simple_1['genotypes'] = genotypes
    
    recessive_comp_simple_2 = {'CHROM':'1', 'POS':'10', 'ALT':'C', 'REF':'T', 'ID':'.', 
                            '1':'1|0', '3':'0|1'}
    genotypes = {'1':genotype.Genotype('1|0'), '3':genotype.Genotype('0|1')}
    recessive_comp_simple_2['genotypes'] = genotypes
    
    
    batch = {'ABC':{'1_5_A_C':recessive_comp_simple_1, '1_10_C_T':recessive_comp_simple_2}}
    batch['haploblocks'] = intervals
    
    check_genetic_models(batch, duo_family, phased=True)
    for gene in batch:
        # pp(batch[gene])
        for variant in batch[gene]:
            pp(batch[gene][variant])
            for ind in batch[gene][variant]['genotypes']:
                print(ind, batch[gene][variant]['genotypes'][ind].__dict__)
    

if __name__ == '__main__':
    main()




try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import pkg_resources

# For making things look nice on pypi:
try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError, RuntimeError):
    long_description = 'Tool for annotating patterns of genetic inheritance in Variant Call Format (VCF) files.'


# with open('README.txt') as file:
#     long_description = file.read()

setup(name='genmod',
    version='1.5.6',
    description='Annotate genetic inheritance models in variant files',
    author = 'Mans Magnusson',
    author_email = 'mans.magnusson@scilifelab.se',
    url = 'http://github.com/moonso/genmod',
    license = 'MIT License',
    install_requires=[
        'ped_parser', 
        'vcf_parser', 
        'pysam', 
        'pytest', 
        'interval_tree', 
        'click'
    ],
    packages = [
        'genmod', 
    ],
    package_data = {
        'genmod': ['annotations/*.db']
    },
    scripts = [
        'scripts/genmod'
    ],
    # entry_points= { "console_scripts" : [
    #     "run_genmod = scripts.run_genmod:main",
    # ]},
    keywords = ['inheritance', 'vcf', 'variants'],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Unix",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
    long_description = long_description,
)


#!/usr/bin/env python
# encoding: utf-8
"""
pair_generator.py

Class that takes a list of objects and return all unordered pairs as a generator.

If only one object? Raise Exception
 
Created by Måns Magnusson on 2013-03-01.
Copyright (c) 2013 __MyCompanyName__. All rights reserved.
"""

from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import argparse

class Pair_Generator(object):
    """Yeilds all unordered pairs from a list of objects as tuples, like (obj_1, obj_2)"""
    def __init__(self, list_of_objects):
        super(Pair_Generator, self).__init__()
        if len(list_of_objects) < 2:
            #TODO raise a proper exception here
            raise SyntaxError('List must include at least 2 objects!. List: %s' % list_of_objects)
        self.list_of_objects = list_of_objects
    
    def generate_pairs(self):
        """Yields all unordered pairs as tuples from the list of objects"""
        for i in range(len(self.list_of_objects)-1):
            for j in range(i+1, len(self.list_of_objects)):
                yield (self.list_of_objects[i], self.list_of_objects[j])
    

def main():
    parser = argparse.ArgumentParser(description="Generate all unordered pairs of a sequence.")
    parser.add_argument('-seq','--sequence', nargs='+', default=['a', 'b', 'c', 'd'], help='A sequence of objects.')
    args = parser.parse_args()
    
    for pairs in Pair_Generator(args.sequence).generate_pairs():
        print(pairs)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
# encoding: utf-8
"""
variant_consumer.py

Class that takes a list of objects and return all unordered pairs as a generator.

If only one object? Raise Exception
 
Created by Måns Magnusson on 2013-03-01.
Copyright (c) 2013 __MyCompanyName__. All rights reserved.
"""

from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import multiprocessing
import sqlite3
import operator
from functools import reduce
from math import log10
from pysam import Tabixfile, asTuple

from pprint import pprint as pp

from genmod import genetic_models, warning

class VariantConsumer(multiprocessing.Process):
    """Yeilds all unordered pairs from a list of objects as tuples, like (obj_1, obj_2)"""
    
    def __init__(self, task_queue, results_queue, family=None, phased=False, vep=False, 
                    cadd_file=None, cadd_1000g=None, thousand_g=None, chr_prefix=False, strict=False, verbosity=False):
        multiprocessing.Process.__init__(self)
        self.task_queue = task_queue
        self.family = family
        self.results_queue = results_queue
        self.verbosity = verbosity
        self.phased = phased
        self.vep = vep
        self.cadd_file = cadd_file
        self.cadd_1000g = cadd_1000g
        self.thousand_g = thousand_g
        self.chr_prefix = chr_prefix
        self.strict = strict
        if self.cadd_1000g:
            self.cadd_1000g = Tabixfile(self.cadd_1000g)
        if self.cadd_file:
            self.cadd_file = Tabixfile(self.cadd_file)
        if self.thousand_g:
            self.thousand_g = Tabixfile(self.thousand_g)
        
        
    def fix_variants(self, variant_batch):
        """Merge the variants into one dictionary, make shure that the compounds are treated right."""
        fixed_variants = {} # dict like {variant_id:variant, ....}
        for feature in variant_batch:
            for variant_id in variant_batch[feature]:
                if variant_id in fixed_variants:
                    # We need to add compound information from different features
                    if len(variant_batch[feature][variant_id].get('Compounds', [])) > 0:
                        fixed_variants[variant_id]['Compounds'] = (
                         dict(list(variant_batch[feature][variant_id]['Compounds'].items()) +
                                    list(fixed_variants[variant_id]['Compounds'].items())))
                else:
                    fixed_variants[variant_id] = variant_batch[feature][variant_id]
        
        return fixed_variants
    
    
    def get_model_score(self, individuals, variant):
        """Return the model score for this variant."""
        model_score = None
        genotype_scores = []
        
        for individual in individuals:
            gt_call = variant['genotypes'].get(individual, None)
            if gt_call:
                if gt_call.genotype_quality > 0:
                    genotype_scores.append(10**-(float(gt_call.genotype_quality)/10))
        if len(genotype_scores) > 0:
            model_score = (str(round(-10*log10(1-reduce(operator.mul, [1-score for score in genotype_scores])))))
        
        return model_score
    
    def get_tabix_record(self, tabix_reader, chrom, start, alt=None):
        """Return the record from a cadd file."""
        record = None
        # CADD values are only for snps:
        cadd_key = int(start)
        try:
            for record in tabix_reader.fetch(str(chrom), cadd_key-1, cadd_key):
                # If Vcf we know there can only be one correct record
                if alt:
                    if record.split('\t')[3] == alt:
                        return record
                else:
                    return record
        except (IndexError, KeyError, ValueError) as e:
            pass
        
        return record
    
    def add_cadd_score(self, variant):
        """Add the CADD relative score to this variant."""
        cadd_record = None
        #Check CADD file(s):
        if self.cadd_file:
            cadd_record = self.get_tabix_record(self.cadd_file, variant['CHROM'], 
                                                variant['POS'], variant['ALT'].split(',')[0])
        # If variant not found in big CADD file check the 1000G file:
        if not cadd_record and self.cadd_1000g:
            cadd_record = self.get_tabix_record(self.cadd_1000g, variant['CHROM'], variant['POS'])
        if cadd_record:
            variant['CADD'] = cadd_record.split('\t')[-1]
        return
    
    def add_frequency(self, variant):
        """Add the thousand genome frequency if present."""
        #Check 1000G frequency:
        thousand_g_record = self.get_tabix_record(self.thousand_g, variant['CHROM'], variant['POS'])
        if thousand_g_record:
            for info in thousand_g_record.split('\t')[7].split(';'):
                info = info.split('=')
                if info[0] == 'AF':
                    variant['1000GMAF'] = info[-1]
        return
    
    
    def make_print_version(self, variant_dict):
        """Get the variants ready for printing"""
        for variant_id in variant_dict:
            variant = variant_dict[variant_id]
            
            if self.chr_prefix:
                variant['CHROM'] = 'chr'+variant_dict[variant_id]['CHROM']
            
            vcf_info = variant_dict[variant_id]['INFO'].split(';')
            
            #Remove the 'Genotypes' post since we will not need them for now
            
            feature_list = variant_dict[variant_id].get('Annotation', [])
            
            if len(variant_dict[variant_id].get('Compounds', [])) > 0:
                #We do not want reference to itself as a compound:
                variant_dict[variant_id]['Compounds'].pop(variant_id, 0)
                compounds_list = list(variant_dict[variant_id]['Compounds'].keys())
                vcf_info.append('Compounds=' + ','.join(compounds_list))
            
            # Check if any genetic models are followed
            model_list = []
            for model in variant_dict[variant_id].get('Inheritance_model',[]):
                if variant_dict[variant_id]['Inheritance_model'][model]:
                    model_list.append(model)
            if len(model_list) > 0:
                vcf_info.append('GeneticModels=' + ','.join(model_list))
                model_score = self.get_model_score(self.family.individuals, variant_dict[variant_id])
                if model_score:
                    if float(model_score) > 0:
                        vcf_info.append('ModelScore=' +  model_score)
            
            # We only want to include annotations where we have a value
            
            if not self.vep:
                if len(feature_list) != 0 and feature_list != ['-']:
                    vcf_info.append('Annotation=' + ','.join(feature_list))
            
            
            if variant.get('CADD', None):
                vcf_info.append('CADD=%s' % str(variant.pop('CADD', '.')))
            
            if variant.get('1000GMAF', None):
                vcf_info.append('1000GMAF=%s' % str(variant.pop('1000GMAF', '.')))
            
            variant_dict[variant_id]['INFO'] = ';'.join(vcf_info)
            
        return
    
    def run(self):
        """Run the consuming"""
        proc_name = self.name
        if self.verbosity:
            print('%s: Starting!' % proc_name)
        while True:
            # A batch is a dictionary on the form {gene:{variant_id:variant_dict}}
            next_batch = self.task_queue.get()
            # if self.verbosity:
                # if self.results_queue.full():
                #     print('Batch results queue Full! %s' % proc_name)
                # if self.task_queue.full():
                #     print('Variant queue full! %s' % proc_name)
            if next_batch is None:
                self.task_queue.task_done()
                if self.verbosity:
                    print('%s: Exiting' % proc_name)
                break
            
            
            if self.family:
                genetic_models.check_genetic_models(next_batch, self.family, self.verbosity, 
                                                    self.phased, self.strict, proc_name)
            # Make shure we only have one copy of each variant:
            fixed_variants = self.fix_variants(next_batch)
            
            for variant_id in fixed_variants:
                if self.cadd_file or self.cadd_1000g:
                    self.add_cadd_score(fixed_variants[variant_id])
                if self.thousand_g:
                    self.add_frequency(fixed_variants[variant_id])
            
            # Now we want to make versions of the variants that are ready for printing.
            self.make_print_version(fixed_variants)
            self.results_queue.put(fixed_variants)
            self.task_queue.task_done()
        return
        
    

def main():
    pass

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# encoding: utf-8
"""
annotation_parser.py

This script will parse a file with intervals in .ccds or .bed format and build one interval tree for each chromosome.
So self.chromosomes will look like:

{'1':intervalTree, '2':intervalTree, ..., 'X':intervalTree}

The intervals represent features that are annotated in the infile.


###UPDATE 2014-04-15###

This parser has been a bit naive since files can have one transcript per line and some one gene per line.
It is hard to write a tool to parse everything. I will now focus on the refGene format that is one transcript per line and each line is like:

from http://genome.ucsc.edu/FAQ/FAQformat.html#format9:

table refFlat
"A gene prediction with additional geneName field."
    (
    string  geneName;           "Name of gene as it appears in Genome Browser."
    string  name;               "Name of gene"
    string  chrom;              "Chromosome name"
    char[1] strand;             "+ or - for strand"
    uint    txStart;            "Transcription start position"
    uint    txEnd;              "Transcription end position"
    uint    cdsStart;           "Coding region start"
    uint    cdsEnd;             "Coding region end"
    uint    exonCount;          "Number of exons"
    uint[exonCount] exonStarts; "Exon start positions"
    uint[exonCount] exonEnds;   "Exon end positions"
    )



Create a family object and its family members from different types of input file
Created by Måns Magnusson on 2013-01-17.
Copyright (c) 2013 __MyCompanyName__. All rights reserved.
"""

from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import argparse
import gzip
import genmod
try:
    import cPickle as pickle
except:
    import pickle

from datetime import datetime
from codecs import open, getreader

from pprint import pprint as pp

from genmod import is_number

from interval_tree import interval_tree

class AnnotationParser(object):
    """Parses a file with family info and creates a family object with individuals."""
    def __init__(self, infile, annotation_type, zipped = False, splice_padding = 2, verbosity=False):
        super(AnnotationParser, self).__init__()
        self.verbosity = verbosity
        self.annotation_type = annotation_type
        
        self.gene_trees = {}# A dictionary with {<chr>:<intervalTree>} the trees are intervals with genes
        self.exon_trees = {}# A dictionary with {<chr>:<intervalTree>} the trees are intervals with exons
        chromosome_stops = {}## A dictionary with information about the last positions on each chromosome:
        infile_name, infile_extension = os.path.splitext(infile)
        
        genes = {}
        exons = {}
        nr_of_genes = 0
        zipped = False
    
        if infile_extension == '.gz':
            zipped = True        
        
        if zipped:
            f = getreader('utf-8')(gzip.open(infile), errors='replace')
        else: 
            f = open(infile, mode='r', encoding='utf-8', errors='replace')
        
        
        if self.annotation_type == 'gene_pred':
            genes,exons = self.gene_pred_parser(f, splice_padding)
        
        elif self.annotation_type == 'gtf':
            genes,exons = self.gtf_parser(f, splice_padding)
        
        elif self.annotation_type == 'ccds':
            genes,exons = self.ccds_parser(f, splice_padding)
        
        elif self.annotation_type == 'bed':
            genes,exons = self.bed_parser(f, splice_padding)
        
        for chrom in dict(genes):
            # prepare the intervals for the tree:
            gene_intervals = []
            exon_intervals = []
            for gene_id in dict(genes[chrom]):
                feature = [genes[chrom][gene_id]['gene_start'],
                        genes[chrom][gene_id]['gene_stop'], gene_id]
                gene_intervals.append(feature)
                
                # Update the end position of the interval
                if genes[chrom][gene_id]['gene_stop'] > chromosome_stops.get(chrom, 0):
                    chromosome_stops[chrom] = genes[chrom][gene_id]['gene_stop'] + 1
                
            genes[chrom] = gene_intervals
            
            for exon in dict(exons.get(chrom,{})):
                exon_intervals.append([exon[0], exon[1], ':'.join(exons[chrom][exon])])
                
            exons[chrom] = exon_intervals
        
        #Build one interval tree for each chromosome:
                
        nr_of_genes = 0
        
        for chrom in genes:
            nr_of_genes += len(genes[chrom])
            self.gene_trees[chrom] = interval_tree.IntervalTree(genes[chrom], 1, chromosome_stops[chrom])
        for chrom in exons:
            if len(exons[chrom]) > 0:
                self.exon_trees[chrom] = interval_tree.IntervalTree(exons[chrom], 1, chromosome_stops[chrom])
            # If no exons found
            else:
                self.exon_trees[chrom] = interval_tree.IntervalTree([[0,0,None]], 0, 0)
        
        if self.verbosity:
            print('Number of genes in annotation file: %s' % nr_of_genes)
                
        return
    
    def add_gene(self, genes, chrom, start, stop, gene_id):
        """Add a gene to a gene dict"""
        if chrom not in genes:
            genes[chrom] = {}
        
        if gene_id in genes[chrom]:
            # If this transcript starts before the previous, update the start of the gene:
            if start < genes[chrom][gene_id]['gene_start']:
                genes[chrom][gene_id]['gene_start'] = start
            # If this transcript ends after the previous update the stop of the gene:
            if stop > genes[chrom][gene_id]['gene_stop']:
                genes[chrom][gene_id]['gene_stop'] = stop
        else:
            genes[chrom][gene_id] = {'gene_start':start, 'gene_stop':stop}
        return
    
    def add_exon(self, exons, chrom, start, stop, feature_id, splice_padding):
        """Add a exon to a exon dict"""
        if chrom not in exons:
            exons[chrom] = {}
        exon = (start - splice_padding, stop + splice_padding)
        if exon in exons[chrom]:
            exons[chrom][exon].append(feature_id)
        else:
            exons[chrom][exon] = [feature_id]
        return
    
    def bed_parser(self, bed_file_handle, splice_padding):
        """Parse a .bed."""
        genes = {}
        exons = {}        
        for line in bed_file_handle:
            if not line.startswith('#') and len(line) > 1:
                line = line.split()
                feature_id = 'None'
                chrom = line[0].lstrip('chr')
                feature_start = int(line[1])
                feature_stop = int(line[2])
                if len(line) > 3:
                    feature_id = line [3]
            
                self.add_gene(genes, chrom, feature_start, feature_stop, feature_id)
                self.add_exon(exons, chrom, feature_start, feature_stop, feature_id, splice_padding)
            
        return genes, exons
    
    def ccds_parser(self, ccds_file_handle, splice_padding):
        """Parse a ccds line"""
        genes = {}
        exons = {}
        
        for line in ccds_file_handle:
            if not line.startswith('#') and len(line) > 1:
                line = line.split('\t')                
                chrom = line[0].lstrip('chr')
                transcript_id = line[1]
                gene_id = line[2]
                if is_number.is_number(line[7]) and is_number.is_number(line[8]):
                    feature_start = int(line[7])
                    feature_stop = int(line[8])
                    #TODO raise exception?
                
                    self.add_gene(genes, chrom, feature_start, feature_stop, gene_id)
                
                    for interval in line[9][1:-1].split(','):
                        boundaries = (interval.split('-'))
                        exon_start = int(boundaries[0].lstrip())
                        exon_stop = int(boundaries[1].lstrip())
        
        return genes, exons
    
    def gtf_parser(self, gtf_file_handle, splice_padding):
        """Parse a gtf file"""
        genes = {}
        exons = {}
        
        for line in gtf_file_handle:
            if not line.startswith('#') and len(line) > 1:
                transcript_id = ''
                gene_id = ''
                gene_name = ''
                
                line = line.split('\t')
                if len(line) < 5:
                    line = line.split()
                chrom = line[0].lstrip('chr')
                if is_number.is_number(line[3]) and is_number.is_number(line[4]):
                    feature_start = int(line[3])
                    feature_stop = int(line[4])
                #TODO Raise exception?
                info_field = line[8].split(';')[:-1]
                for information in info_field:
                    entry = information.split()
                    if entry[0] == 'transcript_id':
                        transcript_id = entry[1][1:-1]
                    if entry[0] == 'gene_id':
                        gene_id = entry[1][1:-1]
                    if entry[0] == 'gene_name':
                        gene_name = entry[1][1:-1]
                
                if line[2] == 'gene':
                    self.add_gene(genes, chrom, feature_start, feature_stop, gene_id)
                 
                elif line[2] == 'exon':
                    self.add_exon(exons, chrom, feature_start, feature_stop, transcript_id, splice_padding)
                
        
        return genes,exons
    
    def gene_pred_parser(self, ref_file_handle, splice_padding):
        """Parse a file in the refGene format, we should add the information about gene or transcript here"""
        genes = {} # A dictionary with {<chr>: {gene_id:{'gene_start':0,'gene_stop':0}}
        exons = {} # A dictionary with {<chr>: {(exon_start, exon_stop, transcript_id_1):[gene_id_1, ..]
        ### Features look like [start,stop, feature_id] ###
        nr_of_lines = 0
        for line in ref_file_handle:
            if not line.startswith('#') and len(line) > 1:
                line = line.split('\t')
                nr_of_lines += 1
                transcript = line[1]
                chrom = line[2].lstrip('chr')
                direction = line[3]
                transc_start = int(line[4])
                transc_stop = int(line[5])
                coding_start = int(line[6])
                coding_stop = int(line[7])
                nr_of_exons = int(line[8])
                exon_starts = [int(boundary) for boundary in line[9].split(',')[:-1]]
                exon_stops = [int(boundary) for boundary in line[10].split(',')[:-1]]
                gene_id = line[12]
                transcript_id = ':'.join([transcript, gene_id])
                                
                self.add_gene(genes, chrom, transc_start, transc_stop, gene_id)
                for i in range(len(exon_starts)):
                    self.add_exon(exons, chrom, exon_starts[i], exon_stops[i], transcript_id, splice_padding)
                
        return genes,exons
        
        

def main():
    parser = argparse.ArgumentParser(description="Parse different kind of annotation files. If no annotation file given, use genes.db and exons.db")
    parser.add_argument('-an','--annotation_file', type=str, nargs=1, default=[None], help='A file with anotations.')
    parser.add_argument('-bed', '--bed', action="store_true", help='Annotation file is in bed format.')
    parser.add_argument('-ccds', '--ccds', action="store_true", help='Annotation file is in ccds format.')
    parser.add_argument('-gtf', '--gtf', action="store_true", help='Annotation file is in gtf format.')
    parser.add_argument('-gene_pred', '--gene_pred', action="store_true", help='Annotation file is in gene pred format. This is the format for the ref_seq and ensembl gene files.')
    parser.add_argument('-v', '--verbose', action="store_true", help='Increase output verbosity.')
    args = parser.parse_args()
    anno_file = args.annotation_file[0]
    gene_trees = {}
    exon_trees = {}
    
    outpath = os.path.join(os.path.split(os.path.dirname(genmod.__file__))[0], 'annotations/')
    gene_db = os.path.join(outpath, 'genes.db')
    exon_db = os.path.join(outpath, 'exons.db')
    
    if args.verbose:
        print(gene_db)
        print(exon_db)
        print('Start parsing annotation!')
        start = datetime.now()
    
    if anno_file:
        file_name, file_extension = os.path.splitext(anno_file)
        zipped = False
        if file_extension == '.gz':
            zipped = True
            file_name, file_extension = os.path.splitext(file_name)
        
        file_type = 'gene_pred'
        if args.bed or file_extension[1:] == 'bed':
            file_type = 'bed'
        if args.ccds or file_extension[1:] == 'ccds':
            file_type = 'ccds'
        if args.gtf or file_extension[1:] == 'gtf':
            file_type = 'gtf'
        if args.gene_pred or file_extension[1:] in ['ref_gene', 'gene_pred']:
            file_type = 'gene_pred'
        
        my_parser = AnnotationParser(anno_file, file_type, zipped=zipped, verbosity=args.verbose)
        
        gene_trees = my_parser.gene_trees
        exon_trees = my_parser.exon_trees
    
    else:
        try:
            with open(gene_db, 'rb') as f:
                gene_trees = pickle.load(f)
            with open(exon_db, 'rb') as g:
                exon_trees = pickle.load(g)
        except FileNotFoundError:
            print('You need to build annotations! See documentation.')
            pass
            # raise FileNotFoundError("You need to make annotations for genmod with an annotation file!")
            #TODO write help line
    
    if args.verbose:
        print('Gene chromosomes:%s' % sorted(gene_trees.keys()))
        print('Exon chromosomes:%s' % sorted(exon_trees.keys()))
        pp(gene_trees.get('1',interval_tree.IntervalTree([[0,0,None]], 1, 1)))
        pp(gene_trees.get('1',interval_tree.IntervalTree([[0,0,None]], 1, 1)).find_range([11900, 11901]))
        pp(gene_trees.get('8',interval_tree.IntervalTree([[0,0,None]], 1, 1)).find_range([6913070, 6913071]))
        pp(exon_trees.get('8',interval_tree.IntervalTree([[0,0,None]], 1, 1)).find_range([6913070, 6913071]))
        pp(gene_trees.get('1',interval_tree.IntervalTree([[0,0,None]], 1, 1)).find_range([721190, 821290]))
        pp(exon_trees.get('1',interval_tree.IntervalTree([[0,0,None]], 1, 1)).find_range([721190, 821290]))
        print('Time to parse file: %s' % str(datetime.now()-start))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
is_number.py

Created by Måns Magnusson on 2013-04-09.
Copyright (c) 2013 __MyCompanyName__. All rights reserved.
"""
from __future__ import print_function
from __future__ import unicode_literals


import sys
import os
import argparse

def is_number(number):
    """Returns true if the string is a number or False otherwise"""
    if type(number) == type(1) or type(number) == type(0.1) or type(number) == type('') or type(u''):
        try:
            float(number)
            return True
        except ValueError:
            return False
        except TypeError:
            return False
    else:
        return False

def main():
    parser = argparse.ArgumentParser(description="Annotate genetic models in variant files..")
    
    parser.add_argument('input', nargs=1, 
        help='give an input to see if it is a number.'
    )
    args = parser.parse_args()
    
    number = args.input[0]
    
    print(is_number(number))


if __name__ == '__main__':
    main()

